@@ -1,5 +1,6 @@
 /*  This file is part of the KDE libraries
  *  Copyright 2013 Aleix Pol Gonzalez <aleixpol@blue-systems.com>
+ *  Copyright 2014 Martin Klapetek <mklapetek@kde.org>
  *
  *  This library is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -19,14 +20,21 @@
  */
 
 #include "kdeplatformfiledialoghelper.h"
+#include "kdeplatformfiledialogbase_p.h"
+#include "kdirselectdialog_p.h"
+
 #include <kfilefiltercombo.h>
 #include <kfilewidget.h>
 #include <klocalizedstring.h>
 #include <kdiroperator.h>
 #include <KUrlComboBox>
+#include <KSharedConfig>
+#include <KWindowConfig>
+#include <KProtocolInfo>
 #include <QVBoxLayout>
 #include <QDialogButtonBox>
 #include <QPushButton>
+#include <QWindow>
 
 #include <QTextStream>
 #include <QEventLoop>
@@ -85,7 +93,7 @@ static QString kde2QtFilter(const QStringList &list, const QString &kde)
 }
 
 KDEPlatformFileDialog::KDEPlatformFileDialog()
-    : QDialog()
+    : KDEPlatformFileDialogBase()
     , m_fileWidget(new KFileWidget(QUrl(), this))
 {
     setLayout(new QVBoxLayout);
@@ -120,6 +128,53 @@ void KDEPlatformFileDialog::selectFile(const QUrl &filename)
     m_fileWidget->setSelection(filename.fileName());
 }
 
+void KDEPlatformFileDialog::setViewMode(QFileDialogOptions::ViewMode view)
+{
+    switch (view) {
+    case QFileDialogOptions::ViewMode::Detail:
+        m_fileWidget->setViewMode(KFile::FileView::Detail);
+        break;
+    case QFileDialogOptions::ViewMode::List:
+        m_fileWidget->setViewMode(KFile::FileView::Simple);
+        break;
+    default:
+        m_fileWidget->setViewMode(KFile::FileView::Default);
+        break;
+    }
+}
+
+void KDEPlatformFileDialog::setFileMode(QFileDialogOptions::FileMode mode)
+{
+    switch (mode) {
+    case QFileDialogOptions::FileMode::AnyFile:
+        m_fileWidget->setMode(KFile::File);
+        break;
+    case QFileDialogOptions::FileMode::ExistingFile:
+        m_fileWidget->setMode(KFile::Mode::File | KFile::Mode::ExistingOnly);
+        break;
+    case QFileDialogOptions::FileMode::Directory:
+        m_fileWidget->setMode(KFile::Mode::Directory);
+        break;
+    case QFileDialogOptions::FileMode::ExistingFiles:
+        m_fileWidget->setMode(KFile::Mode::Files | KFile::Mode::ExistingOnly);
+        break;
+    default:
+        m_fileWidget->setMode(KFile::File);
+        break;
+    }
+}
+
+void KDEPlatformFileDialog::setCustomLabel(QFileDialogOptions::DialogLabel label, const QString &text)
+{
+    if (label == QFileDialogOptions::Accept) { // OK button
+        m_fileWidget->okButton()->setText(text);
+    } else if (label == QFileDialogOptions::Reject) { // Cancel button
+        m_fileWidget->cancelButton()->setText(text);
+    } else if (label == QFileDialogOptions::LookIn) { // Location label
+        m_fileWidget->setLocationLabel(text);
+    }
+}
+
 QString KDEPlatformFileDialog::selectedNameFilter()
 {
     return m_fileWidget->filterWidget()->currentFilter();
@@ -132,7 +187,12 @@ void KDEPlatformFileDialog::selectNameFilter(const QString &filter)
 
 void KDEPlatformFileDialog::setDirectory(const QUrl &directory)
 {
-    m_fileWidget->setStartDir(directory);
+    m_fileWidget->setUrl(directory);
+}
+
+bool KDEPlatformFileDialogHelper::isSupportedUrl(const QUrl& url) const
+{
+    return KProtocolInfo::protocols().contains(url.scheme());
 }
 
 ////////////////////////////////////////////////
@@ -141,6 +201,8 @@ KDEPlatformFileDialogHelper::KDEPlatformFileDialogHelper()
     : QPlatformFileDialogHelper()
     , m_dialog(new KDEPlatformFileDialog)
 {
+    connect(m_dialog, SIGNAL(closed()), SLOT(saveSize()));
+    connect(m_dialog, SIGNAL(finished(int)), SLOT(saveSize()));
     connect(m_dialog, SIGNAL(currentChanged(QUrl)), SIGNAL(currentChanged(QUrl)));
     connect(m_dialog, SIGNAL(directoryEntered(QUrl)), SIGNAL(directoryEntered(QUrl)));
     connect(m_dialog, SIGNAL(fileSelected(QUrl)), SIGNAL(fileSelected(QUrl)));
@@ -152,35 +214,71 @@ KDEPlatformFileDialogHelper::KDEPlatformFileDialogHelper()
 
 KDEPlatformFileDialogHelper::~KDEPlatformFileDialogHelper()
 {
+    saveSize();
     delete m_dialog;
 }
 
 void KDEPlatformFileDialogHelper::initializeDialog()
 {
-    m_dialog->m_fileWidget->setOperationMode(options()->acceptMode() == QFileDialogOptions::AcceptOpen ? KFileWidget::Opening : KFileWidget::Saving);
-    if (options()->windowTitle().isEmpty()) {
-        m_dialog->setWindowTitle(options()->acceptMode() == QFileDialogOptions::AcceptOpen ? i18n("Opening...") : i18n("Saving..."));
+    if (options()->testOption(QFileDialogOptions::ShowDirsOnly)) {
+        KDirSelectDialog *dialog = new KDirSelectDialog(m_dialog->directory());
+        delete m_dialog;
+        m_dialog = dialog;
+        connect(m_dialog, SIGNAL(accepted()), SIGNAL(accept()));
+        connect(m_dialog, SIGNAL(rejected()), SIGNAL(reject()));
     } else {
-        m_dialog->setWindowTitle(options()->windowTitle());
-    }
-    setDirectory(options()->initialDirectory());
+        // needed for accessing m_fileWidget
+        KDEPlatformFileDialog *dialog = qobject_cast<KDEPlatformFileDialog*>(m_dialog);
+        dialog->m_fileWidget->setOperationMode(options()->acceptMode() == QFileDialogOptions::AcceptOpen ? KFileWidget::Opening : KFileWidget::Saving);
+        if (options()->windowTitle().isEmpty()) {
+            dialog->setWindowTitle(options()->acceptMode() == QFileDialogOptions::AcceptOpen ? i18n("Opening...") : i18n("Saving..."));
+        } else {
+            dialog->setWindowTitle(options()->windowTitle());
+        }
+        setDirectory(options()->initialDirectory());
+        dialog->setViewMode(options()->viewMode());
+        dialog->setFileMode(options()->fileMode());
 
-    QStringList filters = options()->mimeTypeFilters();
-    if (!filters.isEmpty()) {
-        m_dialog->m_fileWidget->setMimeFilter(filters);
-    }
+        // custom labels
+        if (options()->isLabelExplicitlySet(QFileDialogOptions::Accept)) { // OK button
+            dialog->setCustomLabel(QFileDialogOptions::Accept, options()->labelText(QFileDialogOptions::Accept));
+        } else if (options()->isLabelExplicitlySet(QFileDialogOptions::Reject)) { // Cancel button
+            dialog->setCustomLabel(QFileDialogOptions::Reject, options()->labelText(QFileDialogOptions::Reject));
+        } else if (options()->isLabelExplicitlySet(QFileDialogOptions::LookIn)) { // Location label
+            dialog->setCustomLabel(QFileDialogOptions::LookIn, options()->labelText(QFileDialogOptions::LookIn));
+        }
 
-    QStringList nameFilters = options()->nameFilters();
-    if (!nameFilters.isEmpty()) {
-        m_dialog->m_fileWidget->setFilter(qt2KdeFilter(nameFilters));
-        if (!options()->initiallySelectedNameFilter().isEmpty()) {
-            selectNameFilter(options()->initiallySelectedNameFilter());
+        // MIME filters
+        QStringList filters = options()->mimeTypeFilters();
+        if (!filters.isEmpty()) {
+            dialog->m_fileWidget->setMimeFilter(filters);
+        }
+
+        // name filters
+        QStringList nameFilters = options()->nameFilters();
+        if (!nameFilters.isEmpty()) {
+            dialog->m_fileWidget->setFilter(qt2KdeFilter(nameFilters));
+            if (!options()->initiallySelectedNameFilter().isEmpty()) {
+                selectNameFilter(options()->initiallySelectedNameFilter());
+            }
+        }
+
+        // overwrite option
+        if (options()->testOption(QFileDialogOptions::FileDialogOption::DontConfirmOverwrite)) {
+            dialog->m_fileWidget->setConfirmOverwrite(false);
         }
     }
 }
 
 void KDEPlatformFileDialogHelper::exec()
 {
+    m_dialog->winId(); // ensure there's a window created
+    KSharedConfig::Ptr conf = KSharedConfig::openConfig();
+    KWindowConfig::restoreWindowSize(m_dialog->windowHandle(), conf->group("FileDialogSize"));
+    // NOTICE: QWindow::setGeometry() does NOT impact the backing QWidget geometry even if the platform
+    // window was created -> QTBUG-40584. We therefore copy the size here.
+    // TODO: remove once this was resolved in QWidget QPA
+    m_dialog->resize(m_dialog->windowHandle()->size());
     m_dialog->exec();
 }
 
@@ -189,6 +287,13 @@ void KDEPlatformFileDialogHelper::hide()
     m_dialog->hide();
 }
 
+void KDEPlatformFileDialogHelper::saveSize()
+{
+    KSharedConfig::Ptr conf = KSharedConfig::openConfig();
+    KConfigGroup group = conf->group("FileDialogSize");
+    KWindowConfig::saveWindowSize(m_dialog->windowHandle(), group);
+}
+
 bool KDEPlatformFileDialogHelper::show(Qt::WindowFlags windowFlags, Qt::WindowModality windowModality, QWindow *parent)
 {
     initializeDialog();
@@ -196,6 +301,8 @@ bool KDEPlatformFileDialogHelper::show(Qt::WindowFlags windowFlags, Qt::WindowMo
     m_dialog->setModal(windowModality != Qt::NonModal);
     if (windowModality == Qt::NonModal) {
         m_dialog->show();
+        KSharedConfig::Ptr conf = KSharedConfig::openConfig();
+        KWindowConfig::restoreWindowSize(m_dialog->windowHandle(), conf->group("FileDialogSize"));
     }
     return true;
 }
